-------------------------------------------------------------------------------
-- ElvUI Chat Tweaks By Lockslap (US, Bleeding Hollow)
-- <Borderline Amazing>, http://ba-guild.com
-- Based on functionality provided by Prat and/or Chatter
-------------------------------------------------------------------------------
local Module	= ElvUI_ChatTweaks:NewModule("Train Alert", "AceEvent-3.0", "AceTimer-3.0")
local L			= LibStub("AceLocale-3.0"):GetLocale("ElvUI_ChatTweaks")
Module.name		= L["Train Alert"]

local db, options
local defaults = {
	profile = {
		summon		= true,
		pet			= "NONE",
		tellRaid	= true,
		raidMsg		= "#name# has placed a #item#!"
	}
}

local selfMsg		= "%s has placed a %s."
local selfPetMsg	= "%s has placed a %s.  %s was summoned."
local playerStr		= "|cff%s%s|r"

local function GetChat()
	if GetNumRaidMembers() > 0 then
		return "RAID"
	elseif GetNumPartyMembers() > 0 then
		return "PARTY"
	else
		return "YELL"
	end
end

local function GetCurrentPet()
	if GetNumCompanions("CRITTER") == 0 then return false end
	for i = 1, GetNumCompanions("CRITTER") do
		-- we just need active flag
		local _, _, _, _, active, _ = GetCompanionInfo("CRITTER", i)
		if active == 1 then
			return i
		end
	end
	return false
end

function Module:SummonPet()
	local id = tonumber(db.pet)
	for i = 1, GetNumCompanions("CRITTER") do
		local _, _, spell = GetCompanionInfo("CRITTER", i)
		if id == spell then
			CallCompanion("CRITTER", i)
			return GetSpellLink(spell)
		end
	end
	return false
end

function Module:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	local _, eType, _, _, source, _, _, _, _, _, _, spellID, spellName = ...
	if eType == "SPELL_CREATE" and spellID == 61031 then
		local _, item = GetItemInfo(44606)
		local _, class = UnitClass(source)
		local player = format(playerStr, PlayerColor(class), source)
		
		-- reset the timer if someone else dropped a trainset
		if self.ResetPet then
			self:CancelTimer(self.ResetPet, true)
			self.ResetPet = nil
		end
	
		-- first, summon the pet
		if db.summon and db.pet ~= "NONE" then 
			local curPet = GetCurrentPet()
			local petLink = self:SummonPet()
			
			if petLink ~= false and curPet ~= false then
				-- schedule a timer in 45 seconds to resummon your previous pet
				self.ResetPet = self:ScheduleTimer(function() CallCompanion("CRITTER", curPet) end, 45)
			end
		end
		
		if db.tellRaid then
			SendChatMessage(db.raidMsg:gsub("#name#", source):gsub("#item#", item), GetChat(), nil)
		end
	end
end

function Module:OnEnable()
	-- we monitor the combat log for the spell create event
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function Module:OnDisable()
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function Module:OnInitialize()
	self.db = ElvUI_ChatTweaks.db:RegisterNamespace("TrainAlert", defaults)
	db = self.db.profile
end

function Module:Info()
	return L["Alerts you when someone puts down a Toy Train Set, with the ability to summon your Lil' XT and tell your raid."]
end

function Module:GetOptions()
	if not options then
		options = {
			summon = {
				type		= "toggle",
				order		= 13,
				name		= L["Summon Pet"],
				desc		= L["Automatically summon a pet that can destroy the train when one is found.\n\nThis will also attempt to resummon the pet you had out previously."],
				get			= function() return db.summon end,
				set			= function(_, value) db.summon = value end,
			},
			pet = {
				type		= "select",
				order		= 14,
				name		= L["Pet to Summon"],
				desc		= L["Choose the pet to summon when a train is dropped."],
				values		= {
					["NONE"]	= L["None"],
					["75906"]	= L["Lil' XT"],
					["93624"]	= L["Landro's Lil' XT"],
				},
				disabled	= function() return not db.summon end,
				get			= function() return db.pet end,
				set			= function(_, value) db.pet = value end,
			},
			tellRaid = {
				type		= "toggle",
				order		= 15,
				name		= L["Tell Raid"],
				desc		= L["Tell your raid (or party) who set down the Toy Train Set."],
				get			= function() return db.tellRaid end,
				set			= function(_, value) db.tellRaid = value end,
			},
			raidMsg = {
				type		= "input",
				order		= 16,
				name		= L["Raid Message"],
				desc		= L["Message to send to the raid.\n\n|cffFA6400Wildcards|r\n|cff00ff00#name#|r - Who placed the Toy Train Set.\n|cff00ff00#item#|r - Item link for Toy Train Set."],
				disabled	= function() return not db.tellRaid end,
				get			= function() return db.raidMsg end,
				set			= function(_, value) db.raidMsg = value end,
			}
		}
	end
	return options
end	